# -*- coding: utf-8 -*-
"""
Created on Thu Jun 22 17:25:29 2023

@author: Elias Adediran
"""

from sklearn.neighbors import KNeighborsRegressor


def idw_interpolation(x, y, z, query_points, k=3, power=2):
    """
    Performs IDW interpolation using scikit-learn's KNeighborsRegressor.

    Args:
        x (array-like): X coordinates of the input points.
        y (array-like): Y coordinates of the input points.
        z (array-like): Z values (observations) corresponding to the input points.
        query_points (array-like): Points to interpolate the Z values.
        k (int): Number of nearest neighbors to consider (default: 3).
        power (int): Power parameter for IDW interpolation (default: 2).

    Returns:
        array: Interpolated Z values corresponding to the query points.
    """
    # Create input and output arrays
    X = list(zip(x, y))
    y = z

    # Create and fit the KNeighborsRegressor model
    model = KNeighborsRegressor(n_neighbors=k, weights='distance', p=power)
    model.fit(X, y)

    # Perform interpolation
    interpolated_values = model.predict(query_points)

    return interpolated_values

# x = [0, 1, 2, 3, 4]
# y = [0, 1, 2, 3, 4]
# z = [0, 1, 4, 9, 16]
# query_points = [(0.5, 0.5), (1.5, 1.5), (2.5, 2.5)]

# interpolated_values = idw_interpolation(x, y, z, query_points)
# print(interpolated_values)