"""
Created on Tue Jun 13 16:39:30 2023

@author: Elias Adediran
"""

import os
import os.path
import sys
from osgeo import gdal, ogr
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import openpyxl
# import cv2
# from scipy.stats import chisquare, ks_2samp
# from matplotlib.lines import Line2D

# append the path of my own written functions
# sys.path.append(r'C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Algorithm_development\Functions')

# import my own functions
from Slope_Roughness import cal_slope, cal_roughness
from Remove_ndv_on_edges import Remove_edge

# Flat 10km
# file_path= r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km_Updated.tif"

# Flat
# file_path= r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88.tif"
# file_path2 = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_16m_NAVD88.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Flat_LA_8m_NAVD88_10km_Updated.tif"

# # Rough
# file_path= r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough\Rough_FL_8m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough\Rough_FL_16m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough\Rough_FL_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Rough_FL_8m_NAVD88_10km.tif"

# # Rough and Slopey
# file_path= r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough_Slopey\Rough_Slopey_ME_8m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough_Slopey\Rough_Slopey_ME_16m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough_Slopey\Rough_Slopey_ME_8m_NAVD88_10km.tif"
file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Rough_Slopey_ME_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Spatial_scale\SS_Rough_Slopey_ME_4m_NAVD88_5km.tif"

# # Slopey
# file_path= r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Slopey\Slopey_MA_8m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Slopey\Slopey_MA_16m_NAVD88.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Slopey\Slopey_MA_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Slopey_MA_8m_NAVD88_10km.tif"


# outdir = "C:/Users/Elias Adediran/OneDrive - USNH/Documents/Thesis/Results/Datasets_Plot/"
# outdir = "C:/Users/eadediran/OneDrive - USNH/Documents/Thesis/Results/Datasets_Plot/Final/"
outdir = "C:/Users/eadediran/OneDrive - USNH/Documents/Thesis/Results/Figures/Slope and roughness histogram plots_updated/"

seabed = 'Flat'
# seabed = 'Rough'
# seabed = 'Rough and Slopey'
# seabed = 'Slopey'


# import the file and get all the necessary info
ds = gdal.Open(file_path)
if not ds:
    raise RuntimeError("Issue in opening the file: %s" % file_path)

print("File was successfully opened: %s" % (file_path,))

depth_gt = ds.GetGeoTransform()
proj = ds.GetProjection()
depth_band = ds.GetRasterBand(1)
ndv= depth_band.GetNoDataValue()
depth= depth_band.ReadAsArray()
res= depth_gt[1]
x_res= depth_gt[1]
y_res= depth_gt[5]
xsize= ds.RasterXSize
ysize= ds.RasterYSize
ds= None

depth= Remove_edge(data=depth, ndv=ndv)

# find index of the nodata value in the data
# x = np.where(depth2== ndv2)

# depth = depth[0:125,0:125]

# convert array to tif
def array_to_tif(array, output_filename):
    driver = gdal.GetDriverByName("GTiff")
    outds = driver.Create(output_filename, xsize=array.shape[1],
                          ysize=array.shape[0], bands=1,
                          eType=gdal.GDT_Float32)
    outds.SetGeoTransform(depth_gt)
    outds.SetProjection(proj)
    outband = outds.GetRasterBand(1)
    outband.WriteArray(array)
    outband.SetNoDataValue(ndv)
    outband.FlushCache()
    return output_filename
    outds = outband = None

depth_tif= array_to_tif(depth, 'depth.tif')


# Calculate slope
slope= cal_slope(depth_tif,'Slope.tif')

# Calculate roughness
roughness= cal_roughness(depth_tif,'roughness.tif')


# plot paramters
parameters = {'axes.labelsize': 10,
          'axes.titlesize': 10,
          'xtick.labelsize': 9,
          'ytick.labelsize': 9}
plt.rcParams.update(parameters)


# Plot Slope Histogram
plt.figure(figsize=(4,2), dpi=600, tight_layout=True)
# plt.hist(slope, bins = 20)
plt.hist(slope)
plt.xlabel('Slope(degree)', fontname='Segoe UI')
plt.ylabel('Frequency', fontname='Segoe UI')
# plt.title('Slope Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution')
ax = plt.gca()
ax.set_xlim(0,20)
plt.axvline(x=1, ls='--', linewidth=1, color='red')
# plt.axvline(x=10, ls='--', linewidth=1, color='blue')
plt.savefig(outdir + 'Slope Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution_axis_test.png', bbox_inches='tight', dpi=600)
plt.interactive(False)
plt.show()

filtered_values = slope[(slope > 2.5) & (slope < 3)]

# Plot Roughness Histogram
plt.figure(figsize=(4,2), dpi=600, tight_layout=True)
plt.hist(roughness, bins = 20)
plt.xlabel('Roughness(degree)', fontname= 'Segoe UI')
plt.ylabel('Frequency', fontname= 'Segoe UI')
# plt.title('Roughness Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution')
ax = plt.gca()
ax.set_xlim(0,1)
plt.axvline(x=0.5, ls='--', linewidth=1, color='red')
# plt.axvline(x=4, ls='--', linewidth=1, color='blue')
plt.savefig(outdir + 'Roughness Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution_axis.png', bbox_inches='tight', dpi=600)
# plt.interactive(False)
# plt.show()


# fig, (ax1, ax2) = plt.subplots(2, figsize=(20,15), tight_layout=True, dpi=300)
# ax1.hist(slope, bins = 20)
# ax1.set_xlabel('Slope(degree)')
# ax1.set_ylabel('Frequency')
# ax1.set_title('Slope Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution')
# ax1.set_xlim([0,30])
# ax2.hist(roughness, bins = 20)
# ax2.set_xlabel('Roughness(degree)')
# ax2.set_ylabel('Frequency')
# ax2.set_title('Roughness Histogram of the '+seabed+ ' Seafloor, '+str(int(res))+'m resolution')
# ax2.set_xlim([0,15])
# plt.show()