"""
Created on Mon Jul 17 14:16:46 2023

@author: Elias Adediran
"""

import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import matplotlib as mpl
from matplotlib_scalebar.scalebar import ScaleBar
import numpy as np
from osgeo import gdal

# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km_Final.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km_Updated.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough\Rough_FL_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Rough_Slopey\Rough_Slopey_ME_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Slopey\Slopey_MA_8m_NAVD88_10km.tif"

file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Flat_LA_8m_NAVD88_10km_Updated.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Rough_FL_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Rough_Slopey_ME_8m_NAVD88_10km.tif"
# file_path = r"C:\Users\eadediran\OneDrive - USNH\Documents\Thesis\Datasets\Final_Datasets\Slopey_MA_8m_NAVD88_10km.tif"

# outdir = "C:/Users/Elias Adediran/OneDrive - USNH/Documents/Thesis/Results/Datasets_Plot/"
outdir = "C:/Users/eadediran/OneDrive - USNH/Documents/Thesis/Results/Figures/"

# location
location = 'LA'
# location = 'FL'
# location = 'ME'
# location = 'MA'

# seabed type
seabed = 'flat'
# seabed = 'Rough'
# seabed = 'Rough and Slopey'
# seabed = 'Slopey'
# seabed = 'mixed'

ds = gdal.Open(file_path)
if not ds:
    raise RuntimeError("Issue in opening the file: %s" % file_path)

print("File was successfully opened: %s" % (file_path,))

depth_gt = ds.GetGeoTransform()
proj = ds.GetProjection()
depth_band = ds.GetRasterBand(1)
ndv = depth_band.GetNoDataValue()
depth = depth_band.ReadAsArray()
res = depth_gt[1]
x_res = depth_gt[1]
y_res = depth_gt[5]
xsize = ds.RasterXSize
ysize = ds.RasterYSize
ds = None

# Get the data extent
ulx = depth_gt[0]
uly = depth_gt[3]
lrx = depth_gt[0] + depth.shape[1] * depth_gt[1]
lry = depth_gt[3] + depth.shape[0] * depth_gt[5]

parameters = {'axes.labelsize': 10,
          'axes.titlesize': 10,
          'xtick.labelsize': 9,
          'ytick.labelsize': 9}
plt.rcParams.update(parameters)

def plot_figure(array, title, cb_label):
    # plt.figure(figsize=(15, 10))
    plt.figure(figsize=(4,3), dpi=600, tight_layout=True)
    output_extent = [ulx, lrx, lry, uly]
    ax = plt.axes()
    plt.ticklabel_format(useOffset=False, style='plain')  # to avoid the display of an offset for the labels
    ax.xaxis.set_major_locator(ticker.MultipleLocator(round(xsize * x_res / 3.5 / 1000) * 1000))  # to set the distance between labels (x axis) and round to nearest thousand
    ax.yaxis.set_major_locator(ticker.MultipleLocator(round(ysize * abs(y_res) / 3.5 / 1000) * 1000))
    array_nan = array.copy().astype('float')
    array_nan[array_nan == ndv] = np.nan  # set the ndv to nan so as to dispace the ndv as white color
    masked_array = np.ma.array(array_nan, mask=np.isnan(array_nan))
    cmap = mpl.cm.jet
    cmap.set_bad('white', 1.)
    img = plt.imshow(masked_array, origin='upper', extent=output_extent, cmap=cmap)
    # plt.title(title, fontsize=10)  # put the filename and the projection as part of the title
    plt.xlabel("Easting (m)")#, fontsize=12)
    plt.ylabel("Northing (m)")#, fontsize=12)
    cb = plt.colorbar(img)  # pass the 'img' object used to create the colorbar
    cb.set_label(cb_label)
    # add_north_arrow(ax)

    # Add the north arrow
    # ax.annotate('N', xy=(0.05, 0.95), xycoords='axes fraction',
    #             fontsize=8, ha='center', va='center', color='black',
    #             bbox=dict(boxstyle='round', fc='none', ec='black', color='black'))

    # Add the scale bar
    scalebar = ScaleBar(1, "m", length_fraction=0.4, height_fraction=0.01, location="lower center")

    # Customize the scale bar properties
    # scalebar.set_font_properties(size=8)
    scalebar.set_color('gray')
    scalebar.set_box_color('gray')
    scalebar.set_box_alpha(0)

    ax.add_artist(scalebar)

    plt.savefig(outdir + title + '.png', bbox_inches='tight', dpi=600)
    plt.interactive(False)
    plt.show()

plot_figure(depth, title= location + ', ' + seabed.capitalize() +' seabed, ' + str(int(x_res)) +'m resolution', cb_label='Bathymetry [meter]')