import numpy as np
import matplotlib.pyplot as plt

from sample_raster_by_percent import sample_raster_by_percent
from compute_proximity_distance import compute_proximity_distance

# This function calculates the number of cells to be removed from the border to correct for the bias of the
# border measurements

def calculate_cell_number(array, no_of_iter, ndv, sampling_percent, plot=True):
    distance_data = np.full((no_of_iter), np.nan, dtype=object)
    percentile = []
    for i in range(no_of_iter):
        sample_data, _ = sample_raster_by_percent(array, sample_percentage=sampling_percent, ndv=ndv)
        distance_data[i] = compute_proximity_distance(array=sample_data, target_value=ndv)
        # Calculate the 99th percentile value
        percentile.append(np.percentile(distance_data[i], 99))

    percentile_median = np.median(percentile)

    # number of cells to be removed from the border
    cell_number = np.int(np.floor(np.median(percentile)))

    # Filter the values within the 99th percentile and plot the histogram
    distance = distance_data[0]
    filtered_distance = distance[distance <= percentile_median]

    if plot:
        plt.ioff()
        plt.figure(figsize=(8,10))
        plt.hist(filtered_distance,bins=10)
        plt.title('Histogram of distances to the nearest depth measurement with 1% sampling density')
        plt.xlabel('Distance from nearest measurement(pixel)')
        plt.ylabel('Frequency')
        plt.show()

    return cell_number