"""
Created on Fri Jul  7 22:04:07 2023

@author: Elias Adediran
"""

from osgeo import gdal
import pandas as pd

def extract_pixel_coordinates(raster_path):
    # Open the raster file
    dataset = gdal.Open(raster_path)

    # Get the geotransformation matrix
    geotransform = dataset.GetGeoTransform()

    # Get the number of rows and columns
    num_rows = dataset.RasterYSize
    num_cols = dataset.RasterXSize

    # Create lists to store the coordinates
    x_coordinates = []
    y_coordinates = []

    # Iterate through each pixel
    for row in range(num_rows):
        for col in range(num_cols):
            # Calculate pixel center coordinates
            x = geotransform[0] + (col + 0.5) * geotransform[1] + (row + 0.5) * geotransform[2]
            y = geotransform[3] + (col + 0.5) * geotransform[4] + (row + 0.5) * geotransform[5]

            # Append the coordinates to the respective lists
            x_coordinates.append(x)
            y_coordinates.append(y)

    # Create a DataFrame with the coordinates
    coordinates_df = pd.DataFrame({'easting': x_coordinates, 'northing': y_coordinates})

    return coordinates_df

# file_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km.tif"
# df = extract_pixel_coordinates(file_path)


# old
# def extract_pixel_coordinates(raster_path):
#     # Open the raster file
#     dataset = gdal.Open(raster_path)
#
#     # Get the geotransformation matrix
#     geotransform = dataset.GetGeoTransform()
#
#     # Get the number of rows and columns
#     num_rows = dataset.RasterYSize
#     num_cols = dataset.RasterXSize
#
#     # Create lists to store the coordinates
#     eastings = []
#     northings = []
#
#     # Iterate through each pixel
#     for row in range(num_rows):
#         for col in range(num_cols):
#             # Convert pixel coordinates to northing and easting
#             x = geotransform[0] + col * geotransform[1] + row * geotransform[2]
#             y = geotransform[3] + col * geotransform[4] + row * geotransform[5]
#
#             # Append the coordinates to the respective lists
#             eastings.append(x)
#             northings.append(y)
#
#     # Create a DataFrame with the coordinates
#     df = pd.DataFrame({'easting': eastings, 'northing': northings})
#
#     # Close the dataset
#     dataset = None
#
#     # Return the DataFrame
#     return df

# raster_path = r"C:\Users\Elias Adediran\OneDrive - USNH\Documents\Thesis\Datasets\Flat\Flat_LA_8m_NAVD88_10km.tif"
# coordinate = extract_pixel_coordinates(raster_path)