from scipy.ndimage import convolve
import numpy as np

def calculate_slope_horn(raster_array, cell_size, window_size):
    if window_size == 3:
    # Horn's finite difference kernel for calculating slope
        kernel_x = np.array([[-1, 0, 1], [-2, 0, 2], [-1, 0, 1]]) / (8.0 * cell_size)
        kernel_y = np.array([[-1, -2, -1], [0, 0, 0], [1, 2, 1]]) / (8.0 * cell_size)

    elif window_size == 5:
        kernel_x = np.array([[-1, -1, 0, 1, 1],
                             [-2, -2, 0, 2, 2],
                             [-3, -3, 0, 3, 3],
                             [-2, -2, 0, 2, 2],
                             [-1, -1, 0, 1, 1]]) / (24 * cell_size)

        kernel_y = np.array([[-1, -2, -3, -2, -1],
                             [-1, -2, -3, -2, -1],
                             [0, 0, 0, 0, 0],
                             [1, 2, 3, 2, 1],
                             [1, 2, 3, 2, 1]]) / (24 * cell_size)

    elif window_size == 7:
        kernel_x = np.array([[-1, -1, -1, 0, 1, 1, 1],
                             [-2, -2, -2, 0, 2, 2, 2],
                             [-3, -3, -3, 0, 3, 3, 3],
                             [-4, -4, -4, 0, 4, 4, 4],
                             [-3, -3, -3, 0, 3, 3, 3],
                             [-2, -2, -2, 0, 2, 2, 2],
                             [-1, -1, -1, 0, 1, 1, 1]]) / (48.0 * cell_size)

        kernel_y = np.array([[-1, -2, -3, -4, -3, -2, -1],
                             [-1, -2, -3, -4, -3, -2, -1],
                             [-1, -2, -3, -4, -3, -2, -1],
                             [0, 0, 0, 0, 0, 0, 0],
                             [1, 2, 3, 4, 3, 2, 1],
                             [1, 2, 3, 4, 3, 2, 1],
                             [1, 2, 3, 4, 3, 2, 1]]) / (48.0 * cell_size)

    else:
        raise ValueError('Window size specified is invalid, please enter 3, 5 or 7')

    # Calculate slope in x and y directions
    slope_x = convolve(raster_array, kernel_x)
    slope_y = convolve(raster_array, kernel_y)

    # Calculate slope magnitude and direction (in radians)
    slope_mag = np.sqrt(slope_x ** 2 + slope_y ** 2)
    slope_direction = np.arctan2(slope_y, slope_x)

    # return np.degrees(slope_mag), np.degrees(slope_direction)
    return np.degrees(slope_mag)
